/*
// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.*/

package org.opengroup.osdu.notification.provider.azure.util;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubHandshakeHandler;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class HandshakeFilterTest {

    @Mock
    private IPubsubRequestBodyExtractor requestBodyExtractor;

    @Mock
    private IPubsubHandshakeHandler handshakeHandler;

    @Mock
    private HttpServletRequest httpServletRequest;

    @Mock
    private HttpServletResponse httpServletResponse;

    @Mock
    private ServletResponse servletResponse;

    @Mock
    private FilterChain filterChain;

    @Mock
    private PrintWriter printWriter;

    @InjectMocks
    private HandshakeFilter sut;

    @Test
    public void should_CallFiltersIfNotHandshake() throws IOException, ServletException {
        doReturn("records-changed").when(httpServletRequest).getServletPath();
        doReturn(false).when(this.requestBodyExtractor).isHandshakeRequest();

        this.sut.doFilter(this.httpServletRequest, this.servletResponse, this.filterChain);

        verify(this.filterChain, times(1)).doFilter(httpServletRequest, servletResponse);
    }

    @Test
    public void should_NotCallFiltersIfHandshake() throws IOException, ServletException {
        doReturn("records-changed").when(httpServletRequest).getServletPath();
        doReturn(true).when(this.requestBodyExtractor).isHandshakeRequest();
        doReturn("handshake response").when(this.handshakeHandler).getHandshakeResponse();
        doReturn(this.printWriter).when(this.httpServletResponse).getWriter();

        this.sut.doFilter(this.httpServletRequest, this.httpServletResponse, this.filterChain);

        verify(this.filterChain, times(0)).doFilter(httpServletRequest, servletResponse);
    }
}
