package org.opengroup.osdu.notification.util;

import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.azure.logging.DependencyLogger;
import org.opengroup.osdu.azure.logging.DependencyLoggingOptions;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.notification.HmacSecret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.azure.util.SubscriberNotificationRequestLogger;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class SubscriberNotificationRequestLoggerTest {

    private final HttpResponse response = new HttpResponse();
    private static final String NOTIFICATION_ID = "test-notification-id";
    private static Subscription subscription;

    @Mock
    private DependencyLogger dependencyLogger;

    @InjectMocks
    private SubscriberNotificationRequestLogger sut;

    @BeforeClass
    public static void setup() {
        subscription = new Subscription();
        subscription.setName("hamc_test_subscription");
        subscription.setPushEndpoint("http://challenge");
        subscription.setDescription("Description");
        subscription.setTopic("records-changed");
        subscription.setNotificationId(NOTIFICATION_ID);
        subscription.setId("id_1");
        subscription.setCreatedBy("test@test.com");
        HmacSecret secret = new HmacSecret();
        secret.setValue("testsecret");
        subscription.setSecret(secret);
    }

    @Test
    public void log_notifySubscriber_success() {
        response.setResponseCode(200);

        this.sut.log(NOTIFICATION_ID, subscription, response);

        ArgumentCaptor<DependencyLoggingOptions> loggingOptionsArgumentCaptor = ArgumentCaptor.forClass(DependencyLoggingOptions.class);
        verify(dependencyLogger, times(1)).logDependency(loggingOptionsArgumentCaptor.capture());
        DependencyLoggingOptions actualLoggingOptions = loggingOptionsArgumentCaptor.getValue();
        verifyDependencyLogging(actualLoggingOptions, "notificationId=test-notification-id subscriptionId=id_1", "http://challenge", 200, true);
    }

    @Test
    public void log_notifySubscriber_failure() {
        response.setResponseCode(403);
        response.setBody("unauthorized");

        this.sut.log(NOTIFICATION_ID, subscription, response);

        ArgumentCaptor<DependencyLoggingOptions> loggingOptionsArgumentCaptor = ArgumentCaptor.forClass(DependencyLoggingOptions.class);
        verify(dependencyLogger, times(1)).logDependency(loggingOptionsArgumentCaptor.capture());
        DependencyLoggingOptions actualLoggingOptions = loggingOptionsArgumentCaptor.getValue();
        verifyDependencyLogging(actualLoggingOptions, "notificationId=test-notification-id subscriptionId=id_1 createdBy=test@test.com response=unauthorized", "http://challenge", 403, false);
    }

    private void verifyDependencyLogging(DependencyLoggingOptions capturedLoggingOptions, String data, String target, int resultCode, boolean success) {
        assertEquals("Notification", capturedLoggingOptions.getType());
        assertEquals("NotifySubscriber", capturedLoggingOptions.getName());
        assertEquals(data, capturedLoggingOptions.getData());
        assertEquals(target, capturedLoggingOptions.getTarget());
        assertEquals(resultCode, capturedLoggingOptions.getResultCode());
        assertEquals(success, capturedLoggingOptions.isSuccess());
    }
}
