# Service Configuration for Baremetal

## Table of Contents <a name="TOC"></a>

* [Service Configuration for Baremetal](#service-configuration-for-baremetal)
  * [Table of Contents <a name="TOC"></a>](#table-of-contents-)
  * [Environment variables](#environment-variables)
    * [Common properties for all environments](#common-properties-for-all-environments)
    * [For Mappers to activate drivers](#for-mappers-to-activate-drivers)
  * [Requirements for requests](#requirements-for-requests)
  * [Configuring mappers Datasources](#configuring-mappers-datasources)
    * [For OQM RabbitMQ](#for-oqm-rabbitmq)
      * [Exchanges and queues configuration](#exchanges-and-queues-configuration)
  * [Interaction with message brokers](#interaction-with-message-brokers)
    * [Specifics of work through PULL subscription](#specifics-of-work-through-pull-subscription)
  * [Keycloak configuration](#keycloak-configuration)

## Environment variables

### Common properties for all environments

| name | value | description | sensitive? | source |
| ---  | ---   | ---         | ---        | ---    |
| `APP_ENTITLEMENTS` | ex `https://entitlements.com/entitlements/v1` | Entitlements API endpoint | no | output of infrastructure deployment |
| `APP_REGISTER` | ex `https://register.com/api/register/v1` | Register API endpoint | no | output of infrastructure deployment |
| `PARTITION_API` | ex `http://localhost:8081/api/partition/v1` | Partition service endpoint | no | - |

**System Environment required to run service**

| name | value | description | sensitive? | source |
| ---  | ---   | ---         | ---        | ---    |
| `SPRING_PROFILES_ACTIVE` | `anthos` | spring active profile | no |
| `OPENID_PROVIDER_CLIENT_ID` | `*****` |  Client id that represents this service and serves to request tokens, example `workload-identity-legal` |yes| - |
| `OPENID_PROVIDER_CLIENT_SECRET` | `*****` | This client secret that serves to request tokens| yes | - |
| `OPENID_PROVIDER_URL` | `https://keycloack.com/auth/realms/master` | URL of OpenID Connect provider, it will be used as `<OpenID URL> + /.well-known/openid-configuration` to auto configure endpoint for token request  | no | - |
| `<AMQP_PASSWORD_ENV_VARIABLE_NAME>` | ex `AMQP_PASS_OSDU` | Amqp password env name, name of that variable not defined at the service level, the name will be received through partition service. Each tenant can have it's own ENV name value, and it must be present in ENV of Notification service | yes | - |
| `<AMQP_ADMIN_PASSWORD_ENV_VARIABLE_NAME>` | ex `AMQP_ADMIN_PASS_OSDU` | Amqp admin password env name, name of that variable not defined at the service level, the name will be received through partition service. Each tenant can have it's own ENV name value, and it must be present in ENV of Notification service | yes | - |
| `REDIS_HOST`                              | ex `127.0.0.1`                             | Redis host                                                                                                                                                                                                                                     | no         | -      |
| `REDIS_PORT`                              | ex `6379`                                  | Redis port                                                                                                                                                                                                                                     | no         | -      |
| `REDIS_PASSWORD`                          | ex ``                                      | Redis password                                                                                                                                                                                                                                 | yes        | -      |
| `REDIS_WITH_SSL`                          | ex `true` or `false`                       | Redis  host ssl config                                                                                                                                                                                                                         | no         |        |
| `RABBITMQ_RETRY_LIMIT`                    | ex `3`                          | RabbitMq retry limit                                                                                                                                                                                                                           | no         |        |
| `RABBITMQ_RETRY_DELAY`                    | ex `1`                          | RabbitMq retry delay                                                                                                                                                                                                                           | no         |        |

### For Mappers to activate drivers

| name      | value     | description                                             |
|-----------|-----------|---------------------------------------------------------|
| OQMDRIVER | pubsub    | to activate **OQM** driver for **Google PubSub**        |
| OQMDRIVER | rabbitmq  | to activate **OQM** driver for **Rabbit MQ**            |

## Testing

After the service has started it should be accessible via a web browser by visiting [http://localhost:8080/api/notification/v1/swagger-ui.html](http://localhost:8080/swagger-ui.html). If the request does not fail, you can then run the integration tests.

### Dependencies needed to run the integration tests

* Java 8
* Maven
* Values for the following environment variables in Config.java

| name                                           | value                                                                           | description                                                                                                                | sensitive?                              | source                                               |
|------------------------------------------------|---------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------------------------|-----------------------------------------|------------------------------------------------------|
| `TEST_DE_OPS_OPENID_PROVIDER_CLIENT_ID`        | `********`                                                                      | Client Id for `DE_OPS_TESTER`                                                                                              | yes                                     | --                                                   |
| `TEST_DE_OPS_OPENID_PROVIDER_CLIENT_SECRET`    | `********`                                                                      |                                                                                                                            | Client secret for `DE_OPS_TESTER`       | yes                                                  | -- |
| `TEST_DE_ADMIN_OPENID_PROVIDER_CLIENT_ID`      | `********`                                                                      | Client Id for `DE_ADMIN_TESTER`                                                                                            | yes                                     | --                                                   |
| `TEST_DE_ADMIN_OPENID_PROVIDER_CLIENT_SECRET`  | `********`                                                                      |                                                                                                                            | Client secret for `DE_ADMIN_TESTER`     | yes                                                  | -- |
| `TEST_DE_EDITOR_OPENID_PROVIDER_CLIENT_ID`     | `********`                                                                      | Client Id for `DE_EDITOR_TESTER`                                                                                           | yes                                     | --                                                   |
| `TEST_DE_EDITOR_OPENID_PROVIDER_CLIENT_SECRET` | `********`                                                                      |                                                                                                                            | Client secret for `DE_EDITOR_TESTER`    | yes                                                  | -- |
| `TEST_NO_ACCESS_OPENID_PROVIDER_CLIENT_ID`     | `********`                                                                      | Client Id for `DE_NO_ACCESS_TESTER`                                                                                        | yes                                     | --                                                   |
| `TEST_NO_ACCESS_OPENID_PROVIDER_CLIENT_SECRET` | `********`                                                                      |                                                                                                                            | Client secret for `DE_NO_ACCESS_TESTER` | yes                                                  | -- |
| `TEST_OPENID_PROVIDER_URL`                     | `https://keycloak.com/auth/realms/osdu`                                         | OpenID provider url                                                                                                        | yes                                     | --                                                   |
| `ENVIRONMENT`                                  | `dev` OR `local` OR `dev_gke`                                                   | Local for running locally with services url's predefined as <http://localhost> , Dev & Dev_Gke is configurable environment | no                                      | -                                                    |
| `HMAC_SECRET`                                  | ex`7a786376626e`                                                                | String in hex , must match pattern ^[a-zA-Z0-9]{8,30}+$ & be in register variable SUBSCRIBER_SECRET                        | yes                                     | -                                                    |
| `REGISTER_BASE_URL`                            | `http://localhost:8081/api/register/v1`                                         | Register service url                                                                                                       | no                                      | -                                                    |
| `NOTIFICATION_BASE_URL`                        | `http://localhost:8080/api/notification/v1/`                                    | Notification service url                                                                                                   | no                                      | -                                                    |
| `CLIENT_TENANT`                                | ex `opendes`                                                                    | Client tenant                                                                                                              | no                                      | -                                                    |
| `OSDU_TENANT`                                  | ex `osdu`                                                                       | osdu tenant                                                                                                                | no                                      | -                                                    |
| `TOPIC_ID`                                     | ex `records-changed`                                                            | PubSub topic id                                                                                                            | no                                      | <https://console.cloud.google.com/cloudpubsub/topic> |
| `REGISTER_CUSTOM_PUSH_URL_HMAC`                | ex `http://localhost:8081/api/register/v1/test/challenge/hmac-integration-test` | Register testing push url                                                                                                  | no                                      | -                                                    |
| `GROUP_ID`                                     | `osdu-gcp.go3-nrg.projects.epam.com`                                            | Group id, used in storage record ACL. Full group will be "data.default.viewers@{{data-partition-id}}.{{group_id}}"         | no                                      | -                                                    |

**Entitlements configuration for integration accounts**

| DE_OPS_TESTER | DE_ADMIN_TESTER | DE_EDITOR_TESTER | DE_NO_ACCESS_TESTER |
| ---  | ---   | ---  | ---   |
|notification.pubsub<br/>service.entitlements.user<br/>users<br/>users.datalake.ops</br>| service.entitlements.user<br/>users<br/>users.datalake.admins</br> | service.entitlements.user<br/>users<br/>users.datalake.editors</br> | service.entitlements.user<br/>users<br/>|

Above variables should be configured in the release pipeline to run integration tests. You should also replace them with proper values if you wish to run tests locally.

### Commands to run tests

* Integration tests are refactored into two pieces: Core and Provider. Core contains business logic for tests and is a dependency for executing the tests from provider module. To build the core module, simply navigate to `notification-test-core` directory and run `mvn clean install`. This will build the core module
* Next, to execute the integration tests, navigate to the provider module and execute `mvn test`

```bash
# (cd testing/notification-test-core/ && mvn clean install)
# Note: this assumes that the environment variables for integration tests as outlined
#       above are already exported in your environment.
$ (cd testing/notification-test-gc/ && mvn clean test)
```

Navigate to notification service's root folder and run all the tests:

```bash
# build + test + install core service code
$ (cd notification-core/ && mvn clean install)
```

## Requirements for requests

Record identifiers cannot contain a space character. At the same time, they may contain a % character, which, when
combined with subsequent numeric characters, may cause the application to misinterpret that combination. For example,
the "%20" combination will be interpreted as a space " " character. To correctly transfer such an identifier, you should
additionally perform the url-encode operation on it. This functionality can be built into the front-end application, or
you can use an online url-encoder tool ( eg.: <https://www.urlencoder.org/>). Thus, having ID "osdu:
work-product-component--WellboreMarkerSet:3D%20Kirchhoff%20DepthMigration" (with %20 combination)
you should url-encode it and request
"osdu%3Awork-product-component--WellboreMarkerSet%3A3D%2520Kirchhoff%2520DepthMigration" instead.

## Configuring mappers DataSources

When using non-Google-Cloud-native technologies, property sets must be defined on the Partition service as part of
PartitionInfo for each Tenant.

### For OQM RabbitMQ

**prefix:** `oqm.rabbitmq`
It can be overridden by:

* through the Spring Boot property `oqm.rabbitmq.partition-properties-prefix`
* environment variable `OQM_RABBITMQ_PARTITION_PROPERTIES_PREFIX``

**PropertySet** (for two types of connection: messaging and admin operations):

| Property | Description |
| --- | --- |
| oqm.rabbitmq.amqp.host | messaging hostname orIP |
| oqm.rabbitmq.amqp.port | - port |
| oqm.rabbitmq.amqp.path | - path |
| oqm.rabbitmq.amqp.username | - username |
| oqm.rabbitmq.amqp.password | - password |
| oqm.rabbitmq.admin.schema | admin host schema |
| oqm.rabbitmq.admin.host | - host name |
| oqm.rabbitmq.admin.port | - port |
| oqm.rabbitmq.admin.path | - path |
| oqm.rabbitmq.admin.username | - username |
| oqm.rabbitmq.admin.password | - password |

<details><summary>Example of a single tenant definition</summary>

```

curl -L -X PATCH 'https://dev.osdu.club/api/partition/v1/partitions/opendes' -H 'data-partition-id: opendes' -H 'Authorization: Bearer ...' -H 'Content-Type: application/json' --data-raw '{
  "properties": {
    "oqm.rabbitmq.amqp.host": {
      "sensitive": false,
      "value": "localhost"
    },
    "oqm.rabbitmq.amqp.port": {
      "sensitive": false,
      "value": "5672"
    },
    "oqm.rabbitmq.amqp.path": {
      "sensitive": false,
      "value": ""
    },
    "oqm.rabbitmq.amqp.username": {
      "sensitive": false,
      "value": "guest"
    },
    "oqm.rabbitmq.amqp.password": {
      "sensitive": true,
      "value": "<AMQP_PASSWORD_ENV_VARIABLE_NAME>" <- (Not actual value, just name of env variable)
    },

     "oqm.rabbitmq.admin.schema": {
      "sensitive": false,
      "value": "http"
    },
     "oqm.rabbitmq.admin.host": {
      "sensitive": false,
      "value": "localhost"
    },
    "oqm.rabbitmq.admin.port": {
      "sensitive": false,
      "value": "9002"
    },
    "oqm.rabbitmq.admin.path": {
      "sensitive": false,
      "value": "/api"
    },
    "oqm.rabbitmq.admin.username": {
      "sensitive": false,
      "value": "guest"
    },
    "oqm.rabbitmq.admin.password": {
      "sensitive": true,
      "value": "<AMQP_ADMIN_PASSWORD_ENV_VARIABLE_NAME>" <- (Not actual value, just name of env variable)
    }
  }
}'

```

</details>

## RabbitMQ configuration

At RabbitMQ should be created set of exchanges and queues.

| topic name                                                                           | subscription name pattern           | description                                                                                                                                                                                  | sensitive? |
|--------------------------------------------------------------------------------------|-------------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|------------|
| `register-subscriber-control`                                                        | `notification-control-sub`          | Register subscriber control topic                                                                                                                                                            | yes        |
| `notification-control-sub-exchange`                                                  | `notification-control-sub`          | Register subscriber control delay topic                                                                                                                                                      | yes        |
| `{topic-name}` e.g. `records-changed`                                                | `notification-{topic-name}-service` | Service topics from [topics.json](https://community.opengroup.org/osdu/platform/system/register/-/blob/master/provider/register-gc/src/main/resources/topics.json) on Register service       | yes        |
| `notification-{topic-name}-service-exchange` e.g. `records-changed-exchange`         | `notification-{topic-name}-service` | Service delay topics from [topics.json](https://community.opengroup.org/osdu/platform/system/register/-/blob/master/provider/register-gc/src/main/resources/topics.json) on Register service | yes        |
| `{topic-name}-publish` e.g. `records-changed-publish`                                | `notification-{topic-name}-publish` | Publish topics in relation 1-on-1 for each service topic                                                                                                                                     | yes        |
| `notification-{topic-name}-publish-exchange` e.g. `records-changed-publish-exchange` | `notification-{topic-name}-publish` | Publish delay topics in relation 1-on-1 for each service topic                                                                                                                               | yes        |


Control topic name can be overridden by:

* through the Spring Boot property `oqm-register-subscriber-control-topic-name`
* environment variable `OQM_REGISTER_SUBSCRIBER_CONTROL_TOPIC_NAME`

![Screenshot](./pics/rabbit.PNG)

### Important. Retry with delay configuration

For every service exchange special delay exchange with postfix `-exchange` 
and type `x-delayed-message` should exist.
Retries and delays configured by `rabbitmqRetryLimit` and `rabbitmqRetryDelay` OQM properties.

Example for `records-changed` topic:

![Screenshot](./pics/Notification-RabbitMQ-Config.png)

## Keycloak configuration

[Keycloak service accounts setup](https://www.keycloak.org/docs/latest/server_admin/#_service_accounts)

Configure Clients. One Client per OSDU service. Set them “confidential”.

![Screenshot](./pics/client.png)

Each Client has embedded Service Account (SA) option. Enable SAs for Clients, make “Authorization enabled”:

![Screenshot](./pics/sa.png)

Add `partition-and-entitlements` scope to `Default Client Scopes` and generate Keys.

Give `client-id` and `client-secret` to services, which should be authorized within the platform.
